unit JLAniBmp;
{==========================================================================}
{                                                                          }
{ animated Bitmap for Delphi 16 & 32 bit                                   }
{                                                                          }
{ written 1996 by Jrg Lingner, Munich/Germany (jlingner@t-online.de)      }
{ Based on Borland's TImage-Component (VCL-source).                        }
{                                                                          }
{ FREEWARE - using it at your own risk                                     }
{==========================================================================}
interface

uses
  WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TAnimatedBitmap = class(TGraphicControl)
  private
    Timer1         :TTimer;
    FBitmap        :TBitmap;
    fActivePicture :Word;
    fColumns       :Word;
    fInterval      :Word;
    fReversePlay   :Boolean;
    fRows          :Word;
    fLeftBmpPos    :Word;
    fTopBmpPos     :Word;
    fPicWidth      :Word;
    fPicHeight     :Word;
    procedure SetBitmap       (aVal:TBitmap);
    procedure SetActivePicture(aVal:Word);
    procedure SetColumns      (aVal:Word);
    procedure SetRows         (aVal:Word);
    procedure SetInterval     (aVal:Word);
    procedure PictureChanged  (Sender:TObject);
    procedure ShowNextPicture (Sender:TObject);
    procedure CalculateVisibleArea;
    function  GetCanvas:TCanvas;
    function  HasBitmap:Boolean;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas       :TCanvas read GetCanvas;
  published
    property ActivePicture:Word    read fActivePicture write SetActivePicture;
    property Columns      :Word    read fColumns       write SetColumns;
    property Interval     :Word    read fInterval      write SetInterval;
    property Bitmap       :TBitmap read fBitmap        write SetBitmap;
    property ReversePlay  :Boolean read fReversePlay   write fReversePlay;
    property Rows         :Word    read fRows          write SetRows;
    property Enabled;
    property ShowHint;
    property Tag;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

{==========================================================================}
implementation

procedure Register;
{==========================================================================}
begin
  RegisterComponents('Freeware',[TAnimatedBitmap]);
end;
{==========================================================================}

constructor TAnimatedBitmap.Create(AOwner:TComponent);
{==========================================================================}
begin
  inherited Create(AOwner);
  fBitmap          := TBitmap.Create;
  fBitmap.OnChange := PictureChanged;

  Height         := 30;
  Width          := 30;
  fReversePlay   := False;
  fColumns       := 1;
  fRows          := 1;
  fInterval      := 0;
  fActivePicture := 0;
  Timer1         := TTimer.Create(Self);
  Timer1.Enabled := False;
  Timer1.OnTimer := ShowNextPicture;
end;
{==========================================================================}

destructor TAnimatedBitmap.Destroy;
{==========================================================================}
begin
  Timer1.Destroy;
  fBitmap.Free;
  inherited Destroy;
end;
{==========================================================================}

procedure TAnimatedBitmap.PictureChanged(Sender:TObject);
{==========================================================================}
begin
  if  (fBitmap.Width = Width) and (fBitmap.Height = Height) then
       ControlStyle := ControlStyle + [csOpaque]
  else ControlStyle := ControlStyle - [csOpaque];
  Invalidate;
end;
{==========================================================================}

function TAnimatedBitmap.HasBitmap:Boolean;
{==========================================================================}
begin
  Result := (fBitmap.Handle <> 0);
end;
{==========================================================================}

function TAnimatedBitmap.GetCanvas:TCanvas;
{==========================================================================}
begin
  Result := fBitmap.Canvas
end;
{==========================================================================}

procedure TAnimatedBitmap.Paint;
{==========================================================================}
var Dest:TRect;
begin
  if csDesigning in ComponentState then
     with inherited Canvas do begin
       Pen.Style := psDash;
       Brush.Style := bsClear;
       Rectangle(0,0,Width,Height);
     end;

  if HasBitmap then begin
     Dest := ClientRect;
     if not fBitmap.Monochrome then
        SetStretchBltMode(inherited Canvas.Handle,STRETCH_DELETESCANS);
     CalculateVisibleArea;
     StretchBlt(inherited Canvas.Handle,Dest.Left,Dest.Top,Dest.Right-Dest.Left,Dest.Bottom-Dest.Top,
                FBitmap.Canvas.Handle,fLeftBmpPos,fTopBmpPos,fPicWidth,fPicHeight,SRCCOPY);
  end;
end;
{==========================================================================}

procedure TAnimatedBitmap.CalculateVisibleArea;
{==========================================================================}
var i,j : Word;
begin
  fPicWidth   := fBitmap.Width  div fColumns;
  fPicHeight  := fBitmap.Height div fRows;

  if fReversePlay then
     for i := Rows-1 downto 0 do begin
         fTopBmpPos := i * fPicHeight;
         for j := Columns-1 downto 0 do begin
             fLeftBmpPos := j * fPicWidth;
             if (i*Columns+j) <= fActivePicture then Exit;
         end;
     end
  else begin
     if fActivePicture >= (fColumns*fRows) then fActivePicture := 0;
     for i := 0 to Rows-1 do begin
         fTopBmpPos := i * fPicHeight;
         for j := 0 to Columns-1 do begin
             fLeftBmpPos := j * fPicWidth;
             if (i*Columns+j) >= fActivePicture then Exit;
         end;
     end;
  end;
end;
{==========================================================================}

procedure TAnimatedBitmap.SetBitmap(aVal:TBitmap);
{==========================================================================}
begin
  fBitmap.Assign(aVal);
end;
{==========================================================================}

procedure TAnimatedBitmap.SetActivePicture(aVal:Word);
{==========================================================================}
begin
  fActivePicture := aVal;
  Paint;
end;
{==========================================================================}

procedure TAnimatedBitmap.SetColumns(aVal:Word);
{==========================================================================}
begin
  if fColumns <> aVal then begin
     fColumns := aVal;
     Invalidate;
  end;
end;
{==========================================================================}

procedure TAnimatedBitmap.SetRows(aVal:Word);
{==========================================================================}
begin
  if fRows <> aVal then begin
     fRows := aVal;
     Invalidate;
  end;
end;
{==========================================================================}

procedure TAnimatedBitmap.SetInterval(aVal:Word);
{==========================================================================}
begin
  if fInterval <> aVal then begin
     fInterval := aVal;
     if aVal = 0 then Timer1.Enabled := False
     else begin
        Timer1.Interval:= aVal;
        Timer1.Enabled := True;
     end;
  end;
end;
{==========================================================================}

procedure TAnimatedBitmap.ShowNextPicture(Sender:TObject);
{==========================================================================}
begin
  if fReversePlay then
     if fActivePicture = 0 then fActivePicture := (fColumns*fRows)-1
                           else Dec(fActivePicture)
  else Inc(fActivePicture);
  Paint;
end;
{==========================================================================}

end.
